package de.jave.eclipse.poormans.quickfixes.popup;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;

import de.jave.eclipse.poormans.quickfixes.util.ProblemIdFactory;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.ui.actions.SelectionConverter;
import org.eclipse.jdt.internal.ui.javaeditor.IJavaAnnotation;
import org.eclipse.jdt.internal.ui.javaeditor.JavaEditor;
import org.eclipse.jdt.internal.ui.text.correction.AssistContext;
import org.eclipse.jdt.internal.ui.text.correction.JavaCorrectionAssistant;
import org.eclipse.jdt.internal.ui.text.correction.ProblemLocation;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jdt.ui.text.java.IInvocationContext;
import org.eclipse.jdt.ui.text.java.IProblemLocation;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.ITextSelection;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationModel;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;

public class ShowProblemIdPopupAction implements IEditorActionDelegate {

  private JavaEditor javaEditor;
  private Clipboard clipboard;

  public final void setActiveEditor(IAction action, IEditorPart targetEditor) {
    if (targetEditor instanceof JavaEditor) {
      javaEditor = (JavaEditor) targetEditor;
    }
    else {
      javaEditor = null;
    }
  }

  private IJavaElement getSelectedObject() throws JavaModelException {
    IJavaElement[] elements = SelectionConverter.codeResolve(javaEditor);
    if (elements != null && elements.length > 0) {
      return elements[0];
    }
    return SelectionConverter.getInput(javaEditor);
  }

  public void run(IAction action) {
    if (javaEditor == null) {
      MessageDialog.openInformation(
          javaEditor.getSite().getShell(),
          "Problem ID",
          "This action only works on Java editors.");
      return;
    }
    ISelection selection = javaEditor.getSelectionProvider().getSelection();
    ICompilationUnit cu = JavaUI.getWorkingCopyManager().getWorkingCopy(javaEditor.getEditorInput());
    IAnnotationModel model = JavaUI.getDocumentProvider().getAnnotationModel(javaEditor.getEditorInput());
    if (!(selection instanceof ITextSelection) || cu == null || model == null) {
      indicateNoProblemFound();
      return;
    }
    IProblemLocation[] problems = getProblem(cu, ((ITextSelection) selection), model);
    if (problems.length == 0) {
      indicateNoProblemFound();
      return;
    }
    IProblemLocation problem = problems[0];
    javaEditor.selectAndReveal(problem.getOffset(), problems[0].getLength());

    final String constantName = ProblemIdFactory.getConstantName(problem.getProblemId());
    final String valueText;
    if (constantName == null) {
      valueText = problem.getProblemId()
          + " (0x"
          + Integer.toHexString(problem.getProblemId()).toUpperCase()
          + ")";
    }
    else {
      valueText = problem.getProblemId() + " (" + constantName + ")";
    }
    MessageDialog.openInformation(javaEditor.getSite().getShell(), "Problem ID", "Problem ID:\r\n"
        + "        "
        + valueText
        + "\r\n\r\nThe constant will be copied to the clipboard.");
    copyToClipboard(String.valueOf(problem.getProblemId()));
  }

  private void indicateNoProblemFound() {
    MessageDialog.openInformation(javaEditor.getSite().getShell(), "Problem ID", "No problem found.");
  }

  private IProblemLocation[] getProblem(ICompilationUnit cu, ITextSelection selection, IAnnotationModel model) {
    AssistContext context = new AssistContext(cu, selection.getOffset(), selection.getLength());
    try {
      boolean goToClosest = selection.getLength() == 0;
      Annotation[] annotations = getAnnotations(selection.getOffset(), goToClosest);
      return collectProposals(context, model, annotations, true, false);
    }
    catch (BadLocationException e) {
      return null;
    }
  }

  private Annotation[] getAnnotations(int offset, boolean goToClosest) throws BadLocationException {
    ArrayList resultingAnnotations = new ArrayList();
    JavaCorrectionAssistant.collectQuickFixableAnnotations(javaEditor, offset, goToClosest, resultingAnnotations);
    return (Annotation[]) resultingAnnotations.toArray(new Annotation[resultingAnnotations.size()]);
  }

  private static IProblemLocation[] collectProposals(
      IInvocationContext context,
      IAnnotationModel model,
      Annotation[] annotations,
      boolean addQuickFixes,
      boolean addQuickAssists) {
    ArrayList problems = new ArrayList();

    // collect problem locations and corrections from marker annotations
    for (int i = 0; i < annotations.length; i++) {
      Annotation curr = annotations[i];
      if (curr instanceof IJavaAnnotation) {
        ProblemLocation problemLocation = getProblemLocation((IJavaAnnotation) curr, model);
        if (problemLocation != null) {
          problems.add(problemLocation);
        }
      }
    }
    IProblemLocation[] problemLocations = (IProblemLocation[]) problems.toArray(new IProblemLocation[problems
        .size()]);
    return problemLocations;
  }

  private static ProblemLocation getProblemLocation(IJavaAnnotation javaAnnotation, IAnnotationModel model) {
    int problemId = javaAnnotation.getId();
    if (problemId != -1) {
      Position pos = model.getPosition((Annotation) javaAnnotation);
      if (pos != null) {
        return new ProblemLocation(pos.getOffset(), pos.getLength(), javaAnnotation); // java problems all handled by the quick assist processors
      }
    }
    return null;
  }

  public void selectionChanged(IAction action, ISelection selection) {
    //nothing to do
  }

  private boolean copyToClipboard(String text) {
    try {
      getClipboard().setContents(new Object[]{ text }, new Transfer[]{ TextTransfer.getInstance() });
      return true;
    }
    catch (SWTError e) {
      if (e.code != DND.ERROR_CANNOT_SET_CLIPBOARD) {
        throw e;
      }
      if (MessageDialog.openQuestion(
          javaEditor.getSite().getShell(),
          "Problem Copying to Clipboard", "There was a problem when accessing the system clipboard. Retry?")) { //$NON-NLS-1$
        return copyToClipboard(text);
      }
    }
    return false;
  }

  private synchronized Clipboard getClipboard() {
    if (clipboard == null) {
      clipboard = new Clipboard(javaEditor.getSite().getShell().getDisplay());
    }
    return clipboard;
  }
}