package de.jave.eclipse.poormans.quickfixes.preferences;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.transform.stream.StreamResult;

import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileInfo;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

public class RegExpQuickFixesExporterImporter {

  public static void doExport(Shell shell, RegExpQuickFix[] fixes) {
    FileDialog dialog = new FileDialog(shell, SWT.SAVE);
    dialog.setText("Export Quick Fixes");
    dialog.setFilterExtensions(new String[]{ "*.xml" });
    dialog.setFileName("quickfixes.xml");
    String path = dialog.open();

    if (path == null) {
      return;
    }

    IFileStore fileStore = EFS.getLocalFileSystem().getStore(new Path(path));
    IFileInfo fileInfo = fileStore.fetchInfo();

    if (fileInfo.getAttribute(EFS.ATTRIBUTE_HIDDEN)) {
      String title = "Error";
      String message = "The selected file cannot be used for export since it is hidden.";
      MessageDialog.openError(shell, title, message);
      return;
    }

    if (fileInfo.exists() && fileInfo.getAttribute(EFS.ATTRIBUTE_READ_ONLY)) {
      String title = "Error";
      String message = "The selected file is read only and can not be written.";
      MessageDialog.openError(shell, title, message);
      return;
    }

    if (!fileInfo.exists() || confirmOverwrite(shell, fileInfo)) {
      OutputStream output = null;
      try {
        output = new BufferedOutputStream(fileStore.openOutputStream(EFS.NONE, null));
        RegExpQuickFixesXmlWriterReader.save(fixes, new StreamResult(output));
      }
      catch (CoreException e) {
        openWriteErrorDialog(shell);
      }
      catch (IOException e) {
        openWriteErrorDialog(shell);
      }
      finally {
        if (output != null) {
          try {
            output.close();
          }
          catch (IOException e) {
            // ignore
          }
        }
      }
    }
  }

  static void addAttribute(Document document, Node node, final String attributeName, String attributeValue) {
    NamedNodeMap attributes = node.getAttributes();
    Attr nameAttr = document.createAttribute(attributeName);
    nameAttr.setValue(attributeValue);
    attributes.setNamedItem(nameAttr);
  }

  private static void openWriteErrorDialog(Shell shell) {
    String title = "Error";
    String message = "Error writing file.";
    MessageDialog.openError(shell, title, message);
  }

  private static boolean confirmOverwrite(Shell shell, IFileInfo fileInfo) {
    return MessageDialog.openQuestion(shell, "Export File Exists", fileInfo.getName()
        + " already exists.\r\n\r\nDo you want to replace it?");
  }

  public static RegExpQuickFix[] doImport(Shell shell) {
    FileDialog dialog = new FileDialog(shell);
    dialog.setText("Import Quick Fixes");
    dialog.setFilterExtensions(new String[]{ "*.xml" });
    String path = dialog.open();

    if (path == null) {
      return null;
    }

    IFileStore fileStore = EFS.getLocalFileSystem().getStore(new Path(path));

    try {
      if (fileStore.fetchInfo().exists()) {
        InputStream input = new BufferedInputStream(fileStore.openInputStream(EFS.NONE, null));
        try {
          return RegExpQuickFixesXmlWriterReader.read(new InputSource(input));
        }
        finally {
          try {
            input.close();
          }
          catch (IOException x) {
            // ignore
          }
        }
      }
    }
    catch (CoreException e) {
      openReadErrorDialog(shell);
    }
    catch (IOException e) {
      openReadErrorDialog(shell);
    }
    return null;
  }

  private static void openReadErrorDialog(Shell shell) {
    String title = "Error";
    String message = "Error reading file.";
    MessageDialog.openError(shell, title, message);
  }
}
