package de.jave.eclipse.poormans.quickfixes.preferences;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import de.jave.eclipse.poormans.quickfixes.regexp.RegExpQuickFix;

import org.eclipse.jface.text.Assert;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class RegExpQuickFixesXmlWriterReader {

  private static final String ATT_REPLACEMENT_FORMAT = "replacementFormat"; //$NON-NLS-1$
  private static final String ATT_PATTERN = "pattern"; //$NON-NLS-1$
  private static final String ATT_PROBLEM_ID = "problemId"; //$NON-NLS-1$
  private static final String ATT_NAME = "name"; //$NON-NLS-1$
  private static final String TAG_REG_EXP_QUICK_FIX = "regExpQuickFix"; //$NON-NLS-1$
  private static final String TAG_POOR_MANS_QUICK_FIXES = "poorMansQuickFixes"; //$NON-NLS-1$
  private static final String ATT_RELEVANCE = "relevance"; //$NON-NLS-1$

  public static void save(final RegExpQuickFix[] fixes, final StreamResult result) throws IOException {
    try {
      final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      final DocumentBuilder builder = factory.newDocumentBuilder();
      final Document document = builder.newDocument();
      final Node root = document.createElement(TAG_POOR_MANS_QUICK_FIXES);
      document.appendChild(root);
      for (int i = 0; i < fixes.length; i++) {
        final RegExpQuickFix data = fixes[i];
        final Node node = document.createElement(TAG_REG_EXP_QUICK_FIX);
        root.appendChild(node);
        RegExpQuickFixesExporterImporter.addAttribute(document, node, ATT_NAME, data.getNameFormat());
        RegExpQuickFixesExporterImporter.addAttribute(document, node, ATT_PROBLEM_ID, String.valueOf(data
            .getProblemId()));
        RegExpQuickFixesExporterImporter.addAttribute(document, node, ATT_PATTERN, data.getPatternString());
        RegExpQuickFixesExporterImporter.addAttribute(document, node, ATT_REPLACEMENT_FORMAT, data
            .getReplacementFormat());
        RegExpQuickFixesExporterImporter.addAttribute(document, node, ATT_RELEVANCE, String.valueOf(data
            .getRelevance()));
      }
      final Transformer transformer = TransformerFactory.newInstance().newTransformer();
      transformer.setOutputProperty(OutputKeys.METHOD, "xml"); //$NON-NLS-1$
      transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8"); //$NON-NLS-1$
      transformer.setOutputProperty(OutputKeys.INDENT, "yes"); //$NON-NLS-1$
      final DOMSource source = new DOMSource(document);
      transformer.transform(source, result);
    }
    catch (final ParserConfigurationException e) {
      Assert.isTrue(false);
    }
    catch (final TransformerException e) {
      if (e.getException() instanceof IOException)
        throw (IOException) e.getException();
      Assert.isTrue(false);
    }
  }

  public static RegExpQuickFix[] read(final InputSource source) throws IOException {
    try {
      final List<RegExpQuickFix> quickFixes = new ArrayList<RegExpQuickFix>();

      final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      final DocumentBuilder parser = factory.newDocumentBuilder();
      final Document document = parser.parse(source);

      final NodeList elements = document.getElementsByTagName(TAG_REG_EXP_QUICK_FIX);
      final int count = elements.getLength();
      for (int i = 0; i != count; i++) {
        final Node node = elements.item(i);
        final NamedNodeMap attributes = node.getAttributes();

        final int problemId = getIntegerValue(attributes, ATT_PROBLEM_ID);
        final String name = getStringValue(attributes, ATT_NAME);
        final String pattern = getStringValue(attributes, ATT_PATTERN);
        final String replacementFormat = getStringValue(attributes, ATT_REPLACEMENT_FORMAT);
        final int relevance = getIntegerValue(attributes, ATT_RELEVANCE, RegExpQuickFix.DEFAULT_RELEVANCE);
        quickFixes.add(new RegExpQuickFix(problemId, pattern, replacementFormat, name, relevance));
      }
      return quickFixes.toArray(new RegExpQuickFix[quickFixes.size()]);
    }
    catch (final ParserConfigurationException e) {
      Assert.isTrue(false);
    }
    catch (final SAXException e) {
      final Throwable t = e.getCause();
      if (t instanceof IOException)
        throw (IOException) t;
      else if (t != null)
        throw new IOException(t.getMessage());
      else
        throw new IOException(e.getMessage());
    }

    return null; // dummy
  }

  private static int getIntegerValue(final NamedNodeMap attributes, final String attributeName)
      throws SAXException {
    final String stringValue = getStringValue(attributes, attributeName);
    try {
      return Integer.parseInt(stringValue);
    }
    catch (final NumberFormatException e) {
      throw new SAXException("Illegal value '" + stringValue + "' for int attribute '" + attributeName + "'.", e); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }
  }

  private static int getIntegerValue(
      final NamedNodeMap attributes,
      final String attributeName,
      final int defaultValue) throws SAXException {
    final String stringValue = getStringValue(attributes, attributeName, String.valueOf(defaultValue));
    try {
      return Integer.parseInt(stringValue);
    }
    catch (final NumberFormatException e) {
      throw new SAXException("Illegal value '" + stringValue + "' for int attribute '" + attributeName + "'.", e); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
    }
  }

  private static String getStringValue(final NamedNodeMap attributes, final String attributeName)
      throws SAXException {
    final String val = getStringValue(attributes, attributeName, null);
    if (val == null) {
      throw new SAXException("Required attribute '" + attributeName + "' missing."); //$NON-NLS-1$ //$NON-NLS-2$
    }
    return val;
  }

  private static String getStringValue(final NamedNodeMap attributes, final String name, final String defaultValue) {
    final Node node = attributes.getNamedItem(name);
    return node == null ? defaultValue : node.getNodeValue();
  }
}